/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


void GetRadioStationsFromFile(bool bParseCustomShoutCast)
{
	KeyValues kv = null;
	char szLocalPath[PLATFORM_MAX_PATH + 1];
	char szParentGenreName[64] = "";
	char szChildGenreName[64] = "";
	char szStationName[128] = "";
	int iParentGenreID = 0;
	int iParentGenreCount = 0;
	int iChildGenreID = 0;
	int iChildGenreCount = 0;
	int iStationCount = 0;
	bool bHasChildren = false;


	g_bStationsParsing = true;
	kv = new KeyValues("Radio Stations");

	if (bParseCustomShoutCast)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsShoutCastCustomFile);
	else if (g_iPlayType == RADIO_PLAY_TYPE_DIRECT)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsDirectBaseFile);
	else if (g_iPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsVolumeBaseFile);
	else if (g_iPlayType == RADIO_PLAY_TYPE_SHOUTCAST)
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szRadioStationsShoutCastBaseFile);
	
	RLogMessage("[SM-RADIO] Importing KeyValues from file \"%s\"", szLocalPath);
	
	if (kv.ImportFromFile(szLocalPath))
	{
		// This is incase we need to append Genre or Station ID's to the Key Values file
		// Keeps the index's persistant across server reboots when admins add new stations or genres without one.
		g_bWriteKVToDisk = false;
		
		kv.GetString("Off Page", g_szRadioOffPage, sizeof(g_szRadioOffPage), "https://radio.dubbeh.net/radiooff.html");
		kv.GetString("Volume Wrappers Base URL", g_szVolumeWrappersBaseURL, sizeof(g_szVolumeWrappersBaseURL), "https://radio.dubbeh.net");
		kv.GetString("Shoutcast Player", g_szShoutCastURL, sizeof(g_szShoutCastURL), "https://radio.dubbeh.net/index.php");
		kv.GetString("Volume Track Query", g_szVolumeControlTrackQueryURL, sizeof(g_szVolumeControlTrackQueryURL), "https://radio.dubbeh.net/trackquery.php");
		
		if (bParseCustomShoutCast)
		{
			g_bCustUseGenres = view_as<bool>(kv.GetNum("Use Genres", 0));
			if (g_bCustUseGenres)
			    g_CustGenres.Init();
			g_CustStations.Init();
		}
		else
		{
			// Put this here to make sure we don't overwrite the ShoutCast playlist version
			g_iShoutCastPlaylistVersion = kv.GetNum("ShoutCast Version", 0);
			g_bUseGenres = view_as<bool>(kv.GetNum("Use Genres", 0));
			if (g_bUseGenres)
			    g_Genres.Init();
			g_Stations.Init();
		}
		
		kv.GotoFirstSubKey(true);
		
		iParentGenreCount = 0;
		iChildGenreCount = 0;
		iStationCount = 0;
		
		do {
			if ((g_bUseGenres && !bParseCustomShoutCast) || (g_bCustUseGenres && bParseCustomShoutCast))
			{
				kv.GetSectionName(szParentGenreName, sizeof(szParentGenreName));
				
				if (szParentGenreName[0])
				{
					iParentGenreID = kv.GetNum("Genre ID", 0);
					bHasChildren = ParseGenreEntry(kv, szParentGenreName, iParentGenreID, 0, bParseCustomShoutCast);
					kv.GotoFirstSubKey(true);
					
					if (bHasChildren)
					{
						// Run over the child genres
						do {
							kv.GetSectionName(szChildGenreName, sizeof(szChildGenreName));
							if (szChildGenreName[0]) {
								iChildGenreID = kv.GetNum("Genre ID", 0);
								ParseGenreEntry(kv, szChildGenreName, iChildGenreID, iParentGenreID, bParseCustomShoutCast);
								iChildGenreCount++;
								
								kv.GotoFirstSubKey(true);
								do {
									if (ParseStationEntry(kv, iChildGenreID, bParseCustomShoutCast))
								    	iStationCount++;
								} while (kv.GotoNextKey(true));
								kv.GoBack();
							}
							else
							{
								RLogMessage("[SM-RADIO] Child genre at index \"%d\" Has no name set. Please correct this error in the config file", iChildGenreCount + 1);
							}
						} while (kv.GotoNextKey(true));
						
						kv.GoBack();
					}
					else
					{
						// Current key has no children - Parse each base key as a station.
						do {
							if (ParseStationEntry(kv, iParentGenreID, bParseCustomShoutCast))
							    iStationCount++;
						} while (kv.GotoNextKey(true));
						
						kv.GoBack();
					}
					
					iParentGenreCount++;
					
				}
				else
				{
					RLogMessage("[SM-RADIO] Parent genre at index \"%d\" Has no name set. Please correct this error in the config file", iParentGenreCount + 1);
				}
			}
			else
			{
				if (kv.GetSectionName(szStationName, sizeof(szStationName)))
				{
					if (ParseStationEntry(kv, 0, bParseCustomShoutCast))
					    iStationCount++;
				}
			}
		} while (kv.GotoNextKey(true));
		
		kv.GoBack();
		
		// Invalid Genre or Station ID's found - Need to update with fixed version
		if (g_bWriteKVToDisk)
		{
			kv.Rewind();
			kv.ExportToFile(szLocalPath);
			g_bWriteKVToDisk = false;
		}
	}
	else
	{	
		if (!bParseCustomShoutCast)
		{
		    g_bPlayListFound = false;
		    
		    // If ShoutCast mode is active and no playlist found - Let's force a download now
		    if (g_iPlayType == RADIO_PLAY_TYPE_SHOUTCAST)
		    {
		    	InitPlaylistDownload(0);
		    }
		}

		g_bStationsParsing = false;
		delete kv;
		RLogMessage("[SM-RADIO] Error: unable to load \"%s\". Make sure you have the file in that location.", szLocalPath);
		return;
	}
	
	RLogMessage("[SM-RADIO] Finishing parsing \"%s\" - Found \"%d\" parent genres, \"%d\" child genres and \"%d\" stations", szLocalPath, iParentGenreCount, iChildGenreCount, iStationCount);
	g_bStationsParsing = false;
	if (!bParseCustomShoutCast)
	    g_bPlayListFound = true;
	delete kv;
}

/*
 * Parse a genre entry from the stations list file
 *
 * returns true on success or false on failure
 *
 */
bool ParseGenreEntry(KeyValues kv, char[] szGenreName, int iGenreID, int iParentGenreID, bool bIsCustomShoutCast)
{
	bool bHasChildren = false;
	
	if (iParentGenreID == 0)
		bHasChildren = view_as<bool>(kv.GetNum("Has Children", 0));
	else
		bHasChildren = false;
	
	// Does the current genre have an ID set?
	if (iGenreID == 0)
	{
		// Genre has no pre-defined index - need to write to disk
		iGenreID = GetURandomInt();
		kv.SetNum("Genre ID", iGenreID);
		g_bWriteKVToDisk = true;
	}
	
	if (bIsCustomShoutCast)
		g_CustGenres.AddEntry(iGenreID, bHasChildren, iParentGenreID, INVALID_HANDLE, szGenreName);
	else
		g_Genres.AddEntry(iGenreID, bHasChildren, iParentGenreID, INVALID_HANDLE, szGenreName);
	
	return bHasChildren;
}

/*
 * Parse a station entry from the stations list file
 *
 * returns true on success or false on failure
 *
 */
bool ParseStationEntry(KeyValues kv, int iGenreID, bool bCustomShoutCastStation)
{
	char szStationName[128] = "";
	char szStreamURL[192] = "";
	int iStationID = 0;
	
	kv.GetSectionName(szStationName, sizeof(szStationName));
	
	if (szStationName[0])
	{
		iStationID = kv.GetNum("Station ID", 0);
		
		// Does the current Station have an ID set?
		if (iStationID == 0)
		{
			// Station has no pre-defined index - need to write to disk
			iStationID = GetURandomInt();
			kv.SetNum("Station ID", iStationID);
			g_bWriteKVToDisk = true;
		}
		
		if (g_iPlayType != RADIO_PLAY_TYPE_SHOUTCAST || bCustomShoutCastStation)
		{
			kv.GetString("Stream URL", szStreamURL, sizeof(szStreamURL), "");
			if (!szStreamURL[0])
			{
				g_bWriteKVToDisk = false;
				RLogMessage("[SM-RADIO] Station \"%s\" has no Stream URL set. Please correct this problem and reload map.", szStationName);
				return false;
			}
		}
		
		if (bCustomShoutCastStation)
			g_CustStations.AddEntry(iStationID, iGenreID, szStationName, szStreamURL);
		else
			g_Stations.AddEntry(iStationID, iGenreID, szStationName, szStreamURL);
		return true;
	}
	else
	{
		g_bWriteKVToDisk = false;
		return false;
	}
}
