/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */

ArrayList g_alGenreIDs = null;
ArrayList g_alGenreHasChildren = null;
ArrayList g_alGenreParentIDs = null;
ArrayList g_alGenreMenuHandles = null;
ArrayList g_alGenreNames = null;

ArrayList g_alCustomGenreIDs = null;
ArrayList g_alCustomGenreHasChildren = null;
ArrayList g_alCustomGenreParentIDs = null;
ArrayList g_alCustomGenreMenuHandles = null;
ArrayList g_alCustomGenreNames = null;

methodmap Genres
{
	property int iCount
	{
		public get()
		{
			return g_alGenreIDs.Length;
		}
	}
	
	public bool Init()
	{
		g_alGenreIDs = CreateArray(8, 0);
		g_alGenreHasChildren = CreateArray(8, 0);
		g_alGenreParentIDs = CreateArray(8, 0);
		g_alGenreMenuHandles = CreateArray(8, 0);
		g_alGenreNames = CreateArray(64, 0);
		if (g_alGenreIDs == null ||
			g_alGenreHasChildren == null ||
			g_alGenreParentIDs == null ||
			g_alGenreMenuHandles == null ||
			g_alGenreNames == null)
			return false;
		else
		    return true;
	}

	public void AddEntry(int iGenreID, bool bHasChildren, int iParentID, Handle hMenu, char[] szGenreName)
	{
		g_alGenreIDs.Push(iGenreID);
		g_alGenreHasChildren.Push(bHasChildren);
		g_alGenreParentIDs.Push(iParentID);
		g_alGenreMenuHandles.Push(hMenu);
		g_alGenreNames.PushString(szGenreName);
	}

	public int GetID(int iGenreIndex)
	{
		return g_alGenreIDs.Get(iGenreIndex);
	}

	public void SetID(int iGenreIndex, int iID)
	{
		g_alGenreIDs.Set(iGenreIndex, iID);
	}

	public bool GetHasChildren(int iGenreIndex)
	{
		return g_alGenreHasChildren.Get(iGenreIndex);
	}

	public void SetHasChildren(int iGenreIndex, bool bHasChildren)
	{
		g_alGenreHasChildren.Set(iGenreIndex, bHasChildren);
	}

	public int GetParentID(int iGenreIndex)
	{
		return g_alGenreParentIDs.Get(iGenreIndex);
	}

	public void SetParentID(int iGenreIndex, int iParentID)
	{
		g_alGenreParentIDs.Set(iGenreIndex, iParentID);
	}

	public Handle GetMenuHandle(int iGenreIndex)
	{
		return g_alGenreMenuHandles.Get(iGenreIndex);
	}

	public void SetMenuHandle(int iGenreIndex, Handle hMenu)
	{
		g_alGenreMenuHandles.Set(iGenreIndex, hMenu);
	}

	public void GetName(int iGenreIndex, char[] szBuffer, int iLength)
	{
		g_alGenreNames.GetString(iGenreIndex, szBuffer, iLength);
	}

	public void SetName(int iGenreIndex, const char[] szBuffer)
	{
		g_alGenreNames.SetString(iGenreIndex, szBuffer);
	}
	
	public bool IsValid(int iGenreIndex)
	{
		if ((iGenreIndex < this.iCount) && (iGenreIndex != -1))
		    return true;
		else
		    return false;
	}
	
	public bool Clear()
	{
		int iSize = this.iCount;
		
		// Free menu handles first
		for (int iIndex = 0; iIndex < iSize; iIndex++)
		{
			CloseHandle(this.GetMenuHandle(iIndex));
		}
		
		delete g_alGenreIDs;
		delete g_alGenreHasChildren;
		delete g_alGenreParentIDs;
		delete g_alGenreMenuHandles;
		delete g_alGenreNames;
	}
	
	public int GetIndexFromID(int iGenreID)
	{
		int iSize = this.iCount;
		
		for (int iIndex = 0; iIndex < iSize; iIndex++)
		{
			if (this.IsValid(iIndex) && this.GetID(iIndex) == iGenreID)
			    return iIndex;
		}
		return -1;
	}
	
	public int FindPrimaryGenreByName(char []szName)
	{
		char szCompareName[64] = "";
		int iSize = this.iCount;
		
		for (int iIndex = 0; iIndex < iSize; iIndex++)
		{
			if (this.GetParentID(iIndex) == 0)
			{
				this.GetName(iIndex, szCompareName, sizeof(szCompareName));
				if (!strcmp(szName, szCompareName, false))
				{
				    return iIndex;
				}
			}
		}
		return -1;
	}
}

methodmap CustomGenres
{
	property int iCount
	{
		public get()
		{
			return g_alCustomGenreIDs.Length;
		}
	}
	
	public bool Init()
	{
		g_alCustomGenreIDs = CreateArray(8, 0);
		g_alCustomGenreHasChildren = CreateArray(8, 0);
		g_alCustomGenreParentIDs = CreateArray(8, 0);
		g_alCustomGenreMenuHandles = CreateArray(8, 0);
		g_alCustomGenreNames = CreateArray(64, 0);
		
		if (g_alCustomGenreIDs == null ||
			g_alCustomGenreHasChildren == null ||
			g_alCustomGenreParentIDs == null ||
			g_alCustomGenreMenuHandles == null ||
			g_alCustomGenreNames == null)
			return false;
		else
		    return true;
	}

	public void AddEntry(int iGenreID, bool bHasChildren, int iParentID, Handle hMenu, char[] szGenreName)
	{
		g_alCustomGenreIDs.Push(iGenreID);
		g_alCustomGenreHasChildren.Push(bHasChildren);
		g_alCustomGenreParentIDs.Push(iParentID);
		g_alCustomGenreMenuHandles.Push(hMenu);
		g_alCustomGenreNames.PushString(szGenreName);
	}

	public int GetID(int iGenreIndex)
	{
		return g_alCustomGenreIDs.Get(iGenreIndex);
	}

	public void SetID(int iGenreIndex, int iID)
	{
		g_alCustomGenreIDs.Set(iGenreIndex, iID);
	}
	
	public bool GetHasChildren(int iGenreIndex)
	{
		return g_alCustomGenreHasChildren.Get(iGenreIndex);
	}

	public void SetHasChildren(int iGenreIndex, bool bHasChildren)
	{
		g_alCustomGenreHasChildren.Set(iGenreIndex, bHasChildren);
	}

	public int GetParentID(int iGenreIndex)
	{
		return g_alCustomGenreParentIDs.Get(iGenreIndex);
	}

	public void SetParentID(int iGenreIndex, int iParentID)
	{
		g_alCustomGenreParentIDs.Set(iGenreIndex, iParentID);
	}

	public Handle GetMenuHandle(int iGenreIndex)
	{
		return g_alCustomGenreMenuHandles.Get(iGenreIndex);
	}

	public void SetMenuHandle(int iGenreIndex, Handle hMenu)
	{
		g_alCustomGenreMenuHandles.Set(iGenreIndex, hMenu);
	}

	public void GetName(int iGenreIndex, char[] szBuffer, int iLength)
	{
		g_alCustomGenreNames.GetString(iGenreIndex, szBuffer, iLength);
	}

	public void SetName(int iGenreIndex, const char[] szBuffer)
	{
		g_alCustomGenreNames.SetString(iGenreIndex, szBuffer);
	}
	
	public bool IsValid(int iGenreIndex)
	{
		if ((iGenreIndex < this.iCount) && (iGenreIndex != -1))
		    return true;
		else
		    return false;
	}
	
	public bool Clear()
	{
		int iSize = this.iCount;
		
		// Free menu handles first
		for (int iIndex = 0; iIndex < iSize; iIndex++)
		{
			CloseHandle(this.GetMenuHandle(iIndex));
		}
		
		delete g_alCustomGenreIDs;
		delete g_alCustomGenreHasChildren;
		delete g_alCustomGenreParentIDs;
		delete g_alCustomGenreMenuHandles;
		delete g_alCustomGenreNames;
	}
	
	public int GetIndexFromID(int iGenreID)
	{
		int iSize = this.iCount;
		
		for (int iIndex = 0; iIndex < iSize; iIndex++)
		{
			if (this.GetID(iIndex) == iGenreID)
			{
			    return iIndex;
			}
		}
		return -1;
	}
	
	public int FindPrimaryGenreByName(char []szName)
	{
		char szCompareName[64] = "";
		int iSize = this.iCount;
		
		for (int iIndex = 0; iIndex < iSize; iIndex++)
		{
			if (this.GetParentID(iIndex) == 0)
			{
				this.GetName(iIndex, szCompareName, sizeof(szCompareName));
				if (!strcmp(szName, szCompareName, false))
				{
				    return iIndex;
				}
			}
		}
		return -1;
	}
}
