/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */

void GetCurretTrack(int iClient)
{
	char szClientStation[32] = "";
	int iStationID = 0;
	Handle hHTTPRequest = INVALID_HANDLE;
	char szQueryURL[128] = "";
	char szCurrentTrack[128] = "";
	bool bIsCustomShoutCast = false;
	char szStreamURL[256] = "";
	int iClientRadioPlayType = 0;
	int iStationIndex = 0;
	
	iStationID = g_Cookies.GetID(iClient);
	bIsCustomShoutCast = g_Cookies.GetIsCustom(iClient);
	iClientRadioPlayType = g_Cookies.GetPlayType(iClient);
	
	if (iStationID > 0 && iClientRadioPlayType == g_iPlayType)
	{
		if (g_iPlayType == RADIO_PLAY_TYPE_SHOUTCAST && !bIsCustomShoutCast)
		{
			iStationIndex = g_Stations.GetIndexFromID(iStationID);
			
			if (g_Stations.IsValid(iStationIndex))
			{
				if (g_Stations.GetLastUpdateTime(iStationIndex) < (GetTime() + TRACK_UPDATE_TIME))
				{
					// Prepare the query URL and execute it
					Format(szQueryURL, sizeof(szQueryURL), "%s?sid=%s", g_szShoutCastQueryURL, szClientStation);
					if (STEAMWORKS_AVAILABLE() && SteamWorks_IsLoaded())
					{
						if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szQueryURL)) != INVALID_HANDLE)
						{
							if (!SteamWorks_SetHTTPRequestContextValue(hHTTPRequest, GetClientSerial(iClient)) || 
								!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnTrackQueryComplete) || 
								!SteamWorks_SendHTTPRequest(hHTTPRequest))
							{
								CReplyToCommand(iClient, "%T", "SteamWorks Query Problem", LANG_SERVER);
								CloseHandle(hHTTPRequest);
							}
						}
					} else {
						CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
					}
				} else {
					g_Stations.GetCurrentTrack(iStationIndex, szCurrentTrack, sizeof(szCurrentTrack));
					CReplyToCommand(iClient, szCurrentTrack);
				}
			} else {
				RLogMessage("[SM-RADIO] Error: Tried to load a station with an invalid index. Please check your stations list file.");
			}
		}
		// Special handling for custom ShoutCast stations
		else if (g_iPlayType == RADIO_PLAY_TYPE_SHOUTCAST && bIsCustomShoutCast)
		{
			iStationIndex = g_CustStations.GetIndexFromID(iStationID);
			
			if (g_CustStations.IsValid(iStationIndex))
			{
				if (g_CustStations.GetLastUpdateTime(iStationIndex) < (GetTime() + TRACK_UPDATE_TIME))
				{
					g_CustStations.GetStreamURL(iStationIndex, szStreamURL, sizeof(szStreamURL));
					
					// Prepare the query URL and execute it
					Format(szQueryURL, sizeof(szQueryURL), "%s?url=%s", g_szVolumeControlTrackQueryURL, szStreamURL);
					
					if (STEAMWORKS_AVAILABLE() && SteamWorks_IsLoaded())
					{
						if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szQueryURL)) != INVALID_HANDLE)
						{
							if (!SteamWorks_SetHTTPRequestContextValue(hHTTPRequest, GetClientSerial(iClient)) || 
								!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnTrackQueryCompleteCustom) || 
								!SteamWorks_SendHTTPRequest(hHTTPRequest))
							{
								CReplyToCommand(iClient, "%T", "SteamWorks Query Problem", LANG_SERVER);
								CloseHandle(hHTTPRequest);
							}
						}
					} else {
						CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
					}
				} else {
					g_CustStations.GetCurrentTrack(iStationIndex, szCurrentTrack, sizeof(szCurrentTrack));
					CReplyToCommand(iClient, szCurrentTrack);
				}
			} else {
				RLogMessage("[SM-RADIO] Error: Tried to load a station with an invalid index. Please check your stations list file.");
			}
		}
		else if (g_iPlayType == RADIO_PLAY_TYPE_VOLUME_CONTROL)
		{
			iStationIndex = g_Stations.GetIndexFromID(iStationID);
			
			if (g_Stations.IsValid(iStationIndex))
			{
				if (g_Stations.GetLastUpdateTime(iStationIndex) < (GetTime() + TRACK_UPDATE_TIME))
				{
					g_Stations.GetStreamURL(iStationIndex, szStreamURL, sizeof(szStreamURL));
					
					// Prepare the query URL and execute it
					Format(szQueryURL, sizeof(szQueryURL), "%s?url=%s", g_szVolumeControlTrackQueryURL, szStreamURL);
					
					if (STEAMWORKS_AVAILABLE() && SteamWorks_IsLoaded())
					{
						if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szQueryURL)) != INVALID_HANDLE)
						{
							if (!SteamWorks_SetHTTPRequestContextValue(hHTTPRequest, GetClientSerial(iClient)) || 
								!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnTrackQueryComplete) || 
								!SteamWorks_SendHTTPRequest(hHTTPRequest))
							{
								CReplyToCommand(iClient, "%T", "SteamWorks Query Problem", LANG_SERVER);
								CloseHandle(hHTTPRequest);
							}
						}
					} else {
						CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
					}
				} else {
					g_Stations.GetCurrentTrack(iStationIndex, szCurrentTrack, sizeof(szCurrentTrack));
					CReplyToCommand(iClient, szCurrentTrack);
				}
			} else {
				RLogMessage("[SM-RADIO] Error: Tried to load a station with an invalid index. Please check your stations list file.");
			}
		} else {
			CReplyToCommand(iClient, "%T", "Track Query Unsupported", LANG_SERVER);
		}
	} else {
		CReplyToCommand(iClient, "Pick a station first to use the track query feature.");
	}
}

public void SteamWorks_OnTrackQueryComplete(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode, any iClientSerial)
{
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK && iClientSerial > 0) {
		SteamWorks_GetHTTPResponseBodyCallback(hRequest, SteamWorks_TrackQueryWebResponse, iClientSerial);
	}
	
	CloseHandle(hRequest);
}

public void SteamWorks_TrackQueryWebResponse(const char[] szResponseBuffer, any iClientSerial)
{
	int iClient = 0;
	int iStationIndex = 0;
	
	iClient = GetClientFromSerial(iClientSerial);
	
	if (iClient && IsClientConnected(iClient) && IsClientInGame(iClient)) {
		iStationIndex = g_Stations.GetIndexFromID(g_Cookies.GetID(iClient));
		if (g_Stations.IsValid(iStationIndex)) {
			g_Stations.SetCurrentTrack(iStationIndex, szResponseBuffer);
			g_Stations.SetLastUpdateTime(iStationIndex, GetTime());
			CPrintToChat(iClient, szResponseBuffer);
		}
	} else {
		RLogMessage("[SM-RADIO] SteamWorks_TrackQueryWebResponse() GetClientSerial returned 0. Maybe the client disconnected.");
	}
}

public void SteamWorks_OnTrackQueryCompleteCustom(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode, any iClientSerial)
{
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK && iClientSerial > 0) {
		SteamWorks_GetHTTPResponseBodyCallback(hRequest, SteamWorks_TrackQueryWebResponseCustom, iClientSerial);
	}
	
	CloseHandle(hRequest);
}

public void SteamWorks_TrackQueryWebResponseCustom(const char[] szResponseBuffer, any iClientSerial)
{
	int iClient = 0;
	int iStationIndex = 0;
	
	iClient = GetClientFromSerial(iClientSerial);
	
	if (iClient && IsClientConnected(iClient) && IsClientInGame(iClient)) {
		iStationIndex = g_CustStations.GetIndexFromID(g_Cookies.GetID(iClient));
		if (g_CustStations.IsValid(iStationIndex)) {
			g_CustStations.SetCurrentTrack(iStationIndex, szResponseBuffer);
			g_CustStations.SetLastUpdateTime(iStationIndex, GetTime());
			CPrintToChat(iClient, szResponseBuffer);
		}
	} else {
		RLogMessage("[SM-RADIO] SteamWorks_TrackQueryWebResponse() GetClientSerial returned 0. Maybe the client disconnected.");
	}
}
